package test;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.math.DoubleMath;
import gr.iti.tools.iknow.IKnow;
import gr.iti.tools.iknow.IKnowConfig;
import gr.iti.tools.iknow.statistics.StatisticsGenerator;
import gr.iti.tools.iknow.statistics.StatisticsManager;
import gr.iti.tools.iknow.statistics.result.ResultPerActivity;
import java.util.LinkedList;
import java.util.List;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *
 * @author gmeditsk
 */
public class TestDistribution {

    private static final Log _logger = LogFactory.getLog(TestDistribution.class);

    public static void main(String[] args) throws Exception {

        // Do not persist results, use local file system
        IKnowConfig.PERSISTS = false;

        /**
         * The triple store to be used for storing observations and activities.
         * AllegroGraph and OWLMIM are currently supported (PERSISTS should be
         * set to true
         */
        //IKnowConfig.ALLEGROGRAPH_SERVER_URL = "http://localhost:10035/";
        //IKnowConfig.OWLIM_SERVER_URL = "http://localhost:8282/openrdf-sesame";
        //IKnowConfig.ALLEGROGRAPH_USERNAME = "";
        //IKnowConfig.ALLEGROGRAPH_PASSWORD = "";
        //IKnowConfig.ALLEGROGRAPH_CATALOG = "";
        /**
         * Path of ontologies.
         *
         * DEFAULT_ONTOLOGIES_PATH is the path to the upper-level ontologies
         * (e.g. the path of the 'ontologies/default' folder of the
         * distribution. ONTOLOGY_DATA_PATH is the path to domain-dependent
         * ontologies (e.g. the path to the 'ontologies/domain' folder of the
         * distribution.
         */
        IKnowConfig.DEFAULT_ONTOLOGIES_PATH = "";
        IKnowConfig.ONTOLOGY_DATA_PATH = "";

        /**
         * The name of domain context descriptor
         */
        IKnowConfig.CONTEXT_DESCRIPTOR = "jws_domainContextDescriptor.rdf";

        /**
         * The path with the dataset to be loaded
         */
        String OBSERVATION_FOLDER = ".../n45/";

        //the number of runs
        int times = 10;

        //the number of days to process
        Integer subset = null;

        //misc variables for collecting and printing perfomance metrics
        ArrayListMultimap<String, Double> recall_results = ArrayListMultimap.create();
        ArrayListMultimap<String, Double> precision_results = ArrayListMultimap.create();
        ArrayListMultimap<String, Double> jrecall_results = ArrayListMultimap.create();
        ArrayListMultimap<String, Double> jprecision_results = ArrayListMultimap.create();

        for (int i = 0; i < times; i++) {
            System.out.println("------------------------------ " + (i + 1) + " --------------------------------");
            IKnow iknow = new IKnow(null);

            iknow.start_on_observations_folder(IKnowConfig.CONTEXT_DESCRIPTOR, OBSERVATION_FOLDER, subset);

            //Initialisation of statistics
            StatisticsManager statisticsManager = iknow.getStatisticsManager();
            LinkedList<StatisticsGenerator> generators = statisticsManager.getGenerators();
            for (StatisticsGenerator g : generators) {
                List<ResultPerActivity> rpaL = g.getResultsPerActivity();
                for (ResultPerActivity rpa : rpaL) {
                    recall_results.put(rpa.getGround().interprets.getLocalName(), rpa.getRecall());
                    precision_results.put(rpa.getGround().interprets.getLocalName(), rpa.getPrecision());
                    jrecall_results.put(rpa.getGround().interprets.getLocalName(), rpa.getjRecall());
                    jprecision_results.put(rpa.getGround().interprets.getLocalName(), rpa.getjPrecision());
                }
            }
        }

        //print performance metrics
        System.out.println(recall_results);
        System.out.println(precision_results);
        System.out.println(jrecall_results);
        System.out.println(jprecision_results);

        System.out.println("");
        System.out.println("--------------------------------------------------------------------------------------");
        System.out.println(String.format("Mean Results (reasoning: %s, times: %d, folder: %s, subset: %d)",
                IKnowConfig.REASONING_MODE, times, OBSERVATION_FOLDER, subset));
        System.out.println("--------------------------------------------------------------------------------------");
        for (String a : recall_results.keySet()) {
            List<Double> recalls = recall_results.get(a);
            List<Double> precisions = precision_results.get(a);
            List<Double> jrecalls = jrecall_results.get(a);
            List<Double> jprecisions = jprecision_results.get(a);
            double mean_recall = 0.0, mean_precision = 0.0, jmean_recall = 0.0, jmean_precision = 0.0;
            if (!recalls.isEmpty() && !precisions.isEmpty()) {
                try {
                    mean_recall = DoubleMath.mean(recalls.iterator());
                    mean_precision = DoubleMath.mean(precisions.iterator());
                    jmean_recall = DoubleMath.mean(jrecalls.iterator());
                    jmean_precision = DoubleMath.mean(jprecisions.iterator());
                } catch (Exception e) {
                }
            }
            System.out.println(String.format("%25s: rec: %.3f, pre: %.3f, jrec: %.3f, jpre %.3f", a, mean_recall, mean_precision, jmean_recall, jmean_precision));
        }

    }

}
